#!/usr/bin/perl --
# jjohn 6/2000
# A CGI interface to a LDAP server. 

use strict;
use CGI         qw/:all *table/;
use CGI::Carp   qw/fatalsToBrowser/;
use CGI::Pretty qw/:all/;
use Net::LDAP;

# first, set up the main objects
my $cgi     = CGI->new();

my $base_dn = 'dc=daisypark, dc=net';  
my $conn    = Net::LDAP->new("ldap.daisypark.net") or
  die "ERROR: Can't connect: $@";

# Make a authenticated connection with the s3cr3t password
$conn->bind( dn       => "cn=Manager, $base_dn",
             password => 's3cr3t',
          );

# Have we been asked to do anything?
my $action = $cgi->param('action');

for ($action){
    my $message = '';
    /search/ && do {
      $message = search(
                        ldap	=> $conn,
                        base_dn	=> $base_dn,
                        cgi	=> $cgi,
                       );
    };

    /modify/ && do {
      $message = modify(
                        ldap	=> $conn,
                        base_dn	=> $base_dn,
                        cgi	=> $cgi,
                      );
    };

    paint( ldap	=> $conn,
           base_dn	=> $base_dn,
           cgi	=> $cgi,
           message	=> $message,
         );
}

$conn->unbind;
exit;

#-------
# Subroutines
#-------
sub paint{
    my %params = @_;

    print
      header,
      start_html( -bgcolor => "#FFFFFF",
                 -title   => ($params{title} ||
                              'View LDAP for Daisypark'),
                ),
      h2( ($params{title} || 'View LDAP for Daisypark') ),
      hr,
      ($params{message} || 'Please perform a search');

    print
      hr,
      start_form,
      '<INPUT TYPE="HIDDEN" NAME="action" VALUE="search">',
      'Search: ',
      textfield( -name => 'search'),
      submit,
      end_form,
      end_html;
}

# ldap_lookup() returns an LDAP entry object
# for the provided search term.
sub ldap_lookup {
    my %params = @_;

    my $criteria = $params{search};
    my $filter;

    # hack, I want to search for everything
    # if I have an empty string
    undef $criteria if $criteria eq '';

    for (qw/c mail sn cn telephonenumber/) {
        # todo: $params{search} needs to escape meta-chars!
        if ( defined $criteria ) {
            $filter .= "($_=*".$criteria."*) ";
        } else {
            $filter .= "($_=*) ";
        }
    }
    $filter = "(| $filter)";

    my $mesg = $params{ldap}->search(
                                     base   => $params{base_dn},
                                     filter => $filter,
                                  );
    if ( $mesg->code ) {
        die "Oops ($filter): ", $mesg->error;
    }

    return $mesg;
 }

 # search() performs a lookup in the LDAP for a given string,
 # returning a nice HTML table.
 
 sub search {
     my %params = @_;

     my $mesg = ldap_lookup( @_,
                             search => ( $params{search} ||
                                         $params{cgi}->param('search')
                                       ),
                          );

     if ( $mesg->count == 0 ) {
         return "No matches found for '$params{search}'";
     }
     my $results;
     $results .= p(small('Matches: ' .
                         b($mesg->count) .
                         ' for term ' .
                         b( $params{cgi}->param('search') )
                 ));
     $results .= start_table( -cellspacing => 0,
                            -cellpadding => 0,
                            );
     $results .= Tr(
                    th({-bgcolor=>'pink'},
                       [qw/Name E-Mail Phone Change/])
                   );
     # add some pretty color every third row
     my $row = 0;
     for my $entry ( $mesg->all_entries ) {

         my $cn = $entry->get('cn')->[0];

         $results .= Tr(
                        {-bgcolor => (!($row%3) ? "#CCCCCC" :"#FFFFFF") },
                        start_form,
                        '<INPUT TYPE="HIDDEN" NAME="action" VALUE="modify">',
                        qq/<INPUT TYPE="HIDDEN" NAME="old_cn" VALUE="$cn">/,
                        td(textfield( {   -name  => 'cn',
                                       -default => $entry->get('cn')
                                  })),
                        td(textfield( {-name     => 'mail',
                                       -default => $entry->get('mail')
                                  })),
                        td(textfield( {-name     => 'telephonenumber',
                                       -default => $entry->get('telephonenumber')
                                  })),
                        td( submit ),
                        end_form,
                      );

         $row++;
      }

      return $results .= end_table;
 }

 sub modify {
   my %params = @_;

   my $old_cn = $params{cgi}->param('old_cn');
   my $mesg = ldap_lookup( @_,
                             search => $old_cn );

   if ( $mesg->count == 0 ) {
       return "Oops: Can't find $old_cn";
   }

   my $cgi = $params{cgi};
   my $entry = $mesg->entry(0); # really need to iterate over results

   # Delete if 'cn' is empty, else modify
   my $report = '';
   if ( $cgi->param('cn') =~ /^\s*$/ ) {
       $entry->delete();
       $report = "Deleted";
   } else {
       $entry->replace(
                       cn	=> $cgi->param('cn'),
                       mail	=> $cgi->param('mail'),
                       telephoneNumber	=> $cgi->param('telephonenumber'),
                      );
       $report = "Updated";
   }

   $entry->update( $params{ldap} );
   return $report . " " . $cgi->param('cn');
 }
